/* voir le fichier qrcodes.h pour des commentaires sur l'utilisation des
 * fonctions */
#include "./qrcodes.h"



/****************************************
 * création et suppression d'un QR code *
 ****************************************/
QRCode alloc_qrcode(int taille) {
    QRCode qr = malloc(taille*sizeof(bit*));
    for (int i=0; i<taille; i++)
        qr[i] = malloc(taille*sizeof(bit));
    return(qr);
}

void free_qrcode(QRCode qr, int taille) {
    for (int i=0; i<taille; i++) free(qr[i]);
    free(qr);
}



/********
 * misc *
 ********/

void print_bits_array(const bit *T, int t) {
  for (int i=0; i<t; i++)
    printf("%d", T[i]);
  printf("\n");
}

void print_bytes_polynomial(const byte *P, int d) {
  int n = 0;
  for (int i=0; i<=d; i++) {
    if (P[i] != 0) {
      if (n>0) printf(" + ");
      n++;
      if (P[i] == 1) {
        if (i==d) printf("1");
        else printf("X^%d", d-i);
      } else {
        printf("%x",P[i]);
        if (i==d) printf(" X^%d",d-i);
      }
    }
  }
  printf("\n");
}

void print_bits_polynomial(const bit *P, int d) {
  print_bytes_polynomial((byte*)P, d);
}

void print_byte(byte c) {
  printf("%x - ", c);
  for (int i=7; i>=0; i--)
    printf("%d", BIT(c,i));
  printf("\n");
}



/*************************************
 * lecture / écriture de fichier PBM *
 *************************************/

/*
 * lire un fichier PBM et obtenir un QR code brut
 *   - image est le nom du fichier
 *   - margin est la taille, en pixel, de la marge
 *   - module_size est la taille, en pixel, des modules
 */
void pbm_to_qrcode_version1(const char *image, int margin, int module_size, QRCode qr) {
  FILE *img = fopen(image, "r");
  if (img == NULL) {
    fprintf(stderr, "Cannot open file \"%s\" for reading. Aborting...\n", image);
    exit(-1);
  }
  int cols, rows;
  bit **T;
  int i,j,x,y;

  T = pbm_readpbm(img, &cols, &rows);

  if (cols < margin + SIZE_V1*module_size  || rows < margin + SIZE_V1*module_size) {
    fprintf(stderr, "PBM file not big enough to contain QR code. Aborting...\n");
    exit(-1);
  }

  for (i=0; i<SIZE_V1; i++) {
    for (j=0; j<SIZE_V1; j++) {
      x = margin+(module_size)*i+module_size/2;
      y = margin+(module_size)*j+module_size/2;
      if (T[x][y] == PBM_WHITE) {
        qr[i][j] = 0;
      } else if (T[x][y] == PBM_BLACK) {
        qr[i][j] = 1;
      } else {
        fprintf(stderr, "Problem while reading PBM file. Aborting...\n");
        exit(-1);
      }
    }
  }
  fclose(img);
}

/*
 * écrire un fichier PBM binaire et obtenir un QR code brut
 *   - image est le nom du fichier
 *   - taille_module est la taille, en pixel, des modules
 */
void qrcode_version1_to_pbm(const char *image, int module_size, QRCode qr) {
  int i,j,x,y;
  int taille = (8+SIZE_V1)*module_size;
  bit **T = malloc(taille*sizeof(bit*));
  for (i=0; i<taille; i++)
    T[i] = malloc(taille*sizeof(bit));

  for (i=0; i<taille; i++) {
    for (j=0; j<taille; j++) {
      if (i<4*module_size || i>=(SIZE_V1+4)*module_size
          || j<4*module_size || j>=(SIZE_V1+4)*module_size) {
        T[i][j] = PBM_WHITE;
      } else {
        x = (i-4*module_size)/module_size;
        y = (j-4*module_size)/module_size;
        if (qr[x][y] == 0) {
          T[i][j] = PBM_WHITE;
        } else if (qr[x][y] == 1) {
          T[i][j] = PBM_BLACK;
        } else {
          fprintf(stderr, "Problem while reading QR code. Aborting...\n");
          exit(-1);
        }
      }
    }
  }
  FILE *img = fopen(image, "w");
  if (img == NULL) {
    fprintf(stderr, "Cannot open file \"%s\" for writing. Aborting...\n", image);
    exit(-1);
  }
  pbm_writepbm(img, T, taille, taille, 1);
  fclose(img);
  for (i=0; i<taille; i++) free(T[i]);
  free(T);
}


/* vim: set tabstop=2 shiftwidth=2 : */
